package com.itsclicking.clickapp.fluttersocketio;

import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterSocketIOPlugin
 */
public class FlutterSocketIOPlugin implements MethodCallHandler {

    private MethodChannel _channel;

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_socket_io");
        channel.setMethodCallHandler(new FlutterSocketIOPlugin(channel));
    }

    private FlutterSocketIOPlugin(MethodChannel channel) {
        _channel = channel;
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        String socketNameSpace = call.argument(SocketIOManager.MethodCallArgumentsName.SOCKET_NAME_SPACE);

        switch (call.method) {
            case SocketIOManager.MethodCallName.SOCKET_CONNECT:
                String callbackStatus = call.argument(SocketIOManager.MethodCallArgumentsName.SOCKET_STATUS_CALLBACK);
                if (!Utils.isNullOrEmpty(socketNameSpace)) {
                    SocketIOManager.getInstance().connectSocket(_channel, socketNameSpace, callbackStatus);
                }
                break;

            case SocketIOManager.MethodCallName.SOCKET_DISCONNECT:
                SocketIOManager.getInstance().disconnect(socketNameSpace);
                break;

            case SocketIOManager.MethodCallName.SOCKET_SUBSCRIBES:
                String socketData = call.argument(SocketIOManager.MethodCallArgumentsName.SOCKET_DATA);
                Map<String, String> map = null;
                if(!Utils.isNullOrEmpty(socketData)) {
                    Utils.log("socketData", socketData);
                    map = Utils.convertJsonToMap(socketData);
                }
                SocketIOManager.getInstance().subscribes(socketNameSpace, map);
                break;

            case SocketIOManager.MethodCallName.SOCKET_UNSUBSCRIBES:
                String jsonData = call.argument(SocketIOManager.MethodCallArgumentsName.SOCKET_DATA);
                Map<String, String> params = null;
                if(!Utils.isNullOrEmpty(jsonData)) {
                    Utils.log("socketData", jsonData);
                    params = Utils.convertJsonToMap(jsonData);
                }
                SocketIOManager.getInstance().subscribes(socketNameSpace, params);
                break;

            case SocketIOManager.MethodCallName.SOCKET_SEND_MESSAGE:
                String event = call.argument(SocketIOManager.MethodCallArgumentsName.SOCKET_EVENT);
                String message = call.argument(SocketIOManager.MethodCallArgumentsName.SOCKET_MESSAGE);

                Utils.log("SocketIOManager", event + message);

                if (!Utils.isNullOrEmpty(event) && !Utils.isNullOrEmpty(message)) {
                    SocketIOManager.getInstance().sendMessage(socketNameSpace, event, message);
                }
                break;

            case SocketIOManager.MethodCallName.SOCKET_DESTROY:
                SocketIOManager.getInstance().destroySocket(socketNameSpace);
                break;

            case SocketIOManager.MethodCallName.SOCKET_DESTROY_ALL:
                SocketIOManager.getInstance().destroyAllSockets();
                break;

            default:
                result.notImplemented();
                break;
        }
    }
}
