package com.itsclicking.clickapp.fluttersocketio;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

public class SocketIOManager implements ISocketIOManager {

    private static final String TAG = "SocketIOManager";
    private static ISocketIOManager mInstance;
    private Map<String, SocketIO> mSockets;


    public synchronized static ISocketIOManager getInstance() {
        if (mInstance == null) {
            mInstance = new SocketIOManager();
        }
        return mInstance;
    }

    private SocketIOManager() {
        mSockets = new HashMap<>();
    }

    private SocketIO getSocket(String socketNameSpace) {
        if(mSockets != null && mSockets.containsKey(socketNameSpace)) {
            return mSockets.get(socketNameSpace);
        }
        return null;
    }

    private boolean isExistedSocketIO(String socketNameSpace) {
        if(!Utils.isNullOrEmpty(socketNameSpace) && mSockets != null && mSockets.size() > 0) {
            return mSockets.containsKey(socketNameSpace);
        }
        return false;
    }

    private void addSocketIO(SocketIO socketIO) {
        if(mSockets == null) {
            mSockets = new HashMap<>();
        }
        if(!isExistedSocketIO(socketIO.getNameSpace())) {
            mSockets.put(socketIO.getNameSpace(), socketIO);
        }
    }

    private void removeSocketIO(String socketNameSpace) {
        if(mSockets != null && isExistedSocketIO(socketNameSpace)) {
            mSockets.remove(socketNameSpace);
        }
    }

    private void removeSocketIO(SocketIO socketIO) {
        if(mSockets != null) {
            mSockets.remove(socketIO.getNameSpace());
        }
    }

    private SocketIO createSocketIO(MethodChannel channel, String socketChannel, String statusCallback) {
        return new SocketIO(channel, socketChannel, statusCallback);
    }

    @Override
    public void connectSocket(MethodChannel channel, String socketNameSpace, final String statusCallback) {
        SocketIO socketIO = getSocket(socketNameSpace);
        if(socketIO == null) {
            socketIO = createSocketIO(channel, socketNameSpace, statusCallback);
        }
        if(socketIO.isConnected()) {
            socketIO.connect();
        }
    }

    @Override
    public void sendMessage(String socketNameSpace, String event, String message) {
        if (isSocketConnected(socketNameSpace) && !Utils.isNullOrEmpty(event)
                && !Utils.isNullOrEmpty(message)) {
            SocketIO socketIO = getSocket(socketNameSpace);
            if(socketIO != null) {
                socketIO.sendMessage(event, message);
            }
        }
    }

    @Override
    public void subscribes(String socketNameSpace, Map<String, String> subscribes) {
        if (isSocketConnected(socketNameSpace)) {
            SocketIO socketIO = getSocket(socketNameSpace);
            if(socketIO != null) {
                socketIO.subscribes(subscribes);
            }
        }
    }

    @Override
    public void unSubscribes(String socketNameSpace, Map<String, String> subscribes) {
        if (isSocketConnected(socketNameSpace)) {
            SocketIO socketIO = getSocket(socketNameSpace);
            if(socketIO != null) {
                socketIO.unSubscribes(subscribes);
            }
        }
    }

    @Override
    public void disconnect(String socketNameSpace) {
        SocketIO socketIO = getSocket(socketNameSpace);
        if(socketIO != null) {
            socketIO.disconnect();
        }
    }

    @Override
    public boolean isSocketConnected(String socketNameSpace) {
        SocketIO socketIO = getSocket(socketNameSpace);
        return socketIO != null && socketIO.isConnected();
    }

    @Override
    public void destroySocket(String socketNameSpace) {
        if(mSockets != null && !mSockets.isEmpty()) {
            SocketIO socketIO = getSocket(socketNameSpace);
            if(socketIO != null) {
                socketIO.destroy();
            }
        }
    }

    @Override
    public void destroyAllSockets() {
        if(mSockets != null && !mSockets.isEmpty()) {
            for(Map.Entry<String, SocketIO> item : mSockets.entrySet()) {
                if(item != null) {
                    SocketIO socket = item.getValue();
                    if(socket != null) {
                        socket.destroy();
                    }
                }
            }
            mSockets.clear();
        }
    }

    public static class MethodCallArgumentsName {
        public static final String SOCKET_NAME_SPACE = "socketNameSpace";
        public static final String SOCKET_STATUS_CALLBACK = "socketStatusCallback";
        public static final String SOCKET_EVENT = "socketEvent";
        public static final String SOCKET_MESSAGE = "socketMessage";
        public static final String SOCKET_DATA = "socketData";
    }

    public static class MethodCallName {
        public static final String SOCKET_CONNECT = "socketConnect";
        public static final String SOCKET_DISCONNECT = "socketDisconnect";
        public static final String SOCKET_SUBSCRIBES = "socketSubcribes";
        public static final String SOCKET_UNSUBSCRIBES = "socketUnsubcribes";
        public static final String SOCKET_SEND_MESSAGE = "socketSendMessage";
        public static final String SOCKET_DESTROY = "socketDestroy";
        public static final String SOCKET_DESTROY_ALL = "socketDestroyAll";
    }
}
