import 'dart:async';

import 'package:flutter/services.dart';
import 'package:flutter_socket_io/iflutter_socket_io.dart';

class FlutterSocketIO implements IFlutterSocketIO {

  static const MethodChannel _channel = const MethodChannel('flutter_socket_io');
  String _socketNameSpace;
  String _socketStatusCallback;

  FlutterSocketIO(String socketNameSpace, String socketStatusCallback,
      Future<dynamic> handler(MethodCall call)) {
    _socketNameSpace = socketNameSpace;
    _socketStatusCallback = socketStatusCallback;
    _channel.setMethodCallHandler(handler);
  }


  @override
  void connectSocket() async {
    await _channel.invokeMethod(
        MethodCallName.SOCKET_CONNECT,
        {
          MethodCallArgumentsName.SOCKET_NAME_SPACE : _socketNameSpace,
          MethodCallArgumentsName.SOCKET_STATUS_CALLBACK : _socketStatusCallback
        });
  }

  /// Map subscribes (Map<String, String>): key is event name, value [optional] is callback event function
  ///
  @override
  void subscribes(Map<String, String> subscribes) async {
    await _channel.invokeMethod(MethodCallName.SOCKET_SUBSCRIBES, subscribes);
  }

  @override
  void sendMessage(String event, String message) async {
    await _channel.invokeMethod(
        MethodCallName.SOCKET_SEND_MESSAGE,
        {
          MethodCallArgumentsName.SOCKET_EVENT : event,
          MethodCallArgumentsName.SOCKET_MESSAGE : message
        });
  }

  /// Map unsubscribes: key is event name, value [optional] is callback event function
  ///
  @override
  void unSubscribes(Map<String, String> unsubscribes) async {
    await _channel.invokeMethod(MethodCallName.SOCKET_UNSUBSCRIBES, unsubscribes);
  }

  @override
  void disconnect() async {
    await _channel.invokeMethod(MethodCallName.SOCKET_DISCONNECT);
  }

}

class MethodCallArgumentsName {
  static final String SOCKET_NAME_SPACE = "socketNameSpace";
  static final String SOCKET_STATUS_CALLBACK = "socketStatusCallback";
  static final String SOCKET_EVENT = "socketEvent";
  static final String SOCKET_MESSAGE = "socketMessage";
}

class MethodCallName {
  static final String SOCKET_CONNECT = "socketConnect";
  static final String SOCKET_DISCONNECT = "socketDisconnect";
  static final String SOCKET_SUBSCRIBES = "socketSubcribes";
  static final String SOCKET_UNSUBSCRIBES = "socketUnsubcribes";
  static final String SOCKET_SEND_MESSAGE = "socketSendMessage";
}
